!
! Sample program of Fortran90 for reading JMA Oean Heat Content (OHC) data
! 
! After execution, this program asks the year to print out and waits for key entering.
! If you put the year (and "Enter"), 
! this program reads JMA OHC data in ASCII files, and prints out the (179.5E, 0.5N) value of the year.
! 
! 2020.02 Japan Meteorological Agency
! 
!
program ohcsample
  implicit none
  
  ! Main variables
  ! 
  ! * GRID Configuration
  ! imx : number of grids in longitude
  ! jmx : number of grids in latitude
  ! 
  ! * Analysed fields
  ! ohc(imx,jmx) : Ocean Heat Content anomalies of each grid
  ! err(imx,jmx) : Analysis errors of each grid
  ! 
  ! * File name and format
  ! flohc : OHC data file name template ('YYY1' and 'YYY2' are replaced by years)
  ! flerr : Error data file name template ('YYY1' and 'YYY2' are replaced by years)
  ! fmt   : Data format in ASCII file
  ! unit  : Data unit in [J]
  ! xmiss : Missing value (read from header)
  ! 
  ! * Years
  ! iyear      : Year to print out
  ! year       : Year read in header line
  ! start_year : First year in all data
  ! nyrpack    : Number of years packed in one file
  ! 
  ! ohc(imx,jmx) and err(imx,jmx) indicates the grids centered at as follows
  ! (1,1)   = (0.5E,89.5N)  , (2,1)   = (1.5E,89.5N) ,,, (360,1)   = (0.5W,89.5N)
  ! (1,2)   = (0.5E,88.5N)  , (2,2)   = (1.5E,88.5N) ,,, (360,2)   = (0.5W,88.5N)
  ! ,,,
  ! (1,180) = (0.5E,89.5S)  , (2,180) = (1.5E,89.5S) ,,, (360,180) = (0.5W,89.5S)
  !

! Variables
  integer :: i,j,i2,j2
  
  integer,parameter :: imx=360,jmx=180
  double precision :: ohc(imx,jmx)
  double precision :: err(imx,jmx)
  character*20 :: fmt='(360f8.4)'
  double precision :: unit=1.0D19
  
  integer :: iyear,year
  integer :: mtohc=10,mterr=11
  integer :: iosohc,ioserr
  character*40 :: flohc='ohc_YYY1_YYY2'
  integer :: iptohc1,iptohc2
  character*40 :: flerr='ohc_err_YYY1_YYY2'
  integer :: ipterr1,ipterr2
  integer :: iyear_first,iyear_last
  integer,parameter :: nyrpack=10
  integer,parameter :: start_year=1955
  
  character*200 :: header1,header2
  integer :: iptmiss
  double precision :: xmiss
  
  
! Set File name configuration
  iptohc1=index(flohc,'YYY1')
  iptohc2=index(flohc,'YYY2')
  ipterr1=index(flerr,'YYY1')
  ipterr2=index(flerr,'YYY2')
  
! Ask the target year
  write(*,*) 'Target year?'
  read(*,*) iyear

! Open OHC and error files
  iyear_first=(iyear/nyrpack)*nyrpack
  iyear_last=iyear_first+nyrpack-1
  if(iyear_last.eq.1959) iyear_first=start_year
  
  write(flohc(iptohc1:iptohc1+3),'(i4.4)') iyear_first
  write(flohc(iptohc2:iptohc2+3),'(i4.4)') iyear_last
  write(flerr(ipterr1:ipterr1+3),'(i4.4)') iyear_first
  write(flerr(ipterr2:ipterr2+3),'(i4.4)') iyear_last

  open(mtohc,file=flohc,access='sequential',form='formatted',status='old',iostat=iosohc)
  open(mterr,file=flerr,access='sequential',form='formatted',status='old',iostat=ioserr)
  if(iosohc.ne.0.or.ioserr.ne.0) then
!!!! In the most recent file name, the last year is indicated as 'last'.
    write(flohc(iptohc2:iptohc2+3),'(a4)') 'last'
    write(flerr(ipterr2:ipterr2+3),'(a4)') 'last'
    open(mtohc,file=flohc,access='sequential',form='formatted',status='old',iostat=iosohc)
    open(mterr,file=flerr,access='sequential',form='formatted',status='old',iostat=ioserr)
    if(iosohc.ne.0.or.ioserr.ne.0) then
      write(6,*) 'No data files'
      stop
    else
      write(*,*) flohc,' is opened'
      write(*,*) flerr,' is opened'
    endif
  else
    write(*,*) flohc,' is opened'
    write(*,*) flerr,' is opened'
  endif
  
! Read data
  do
!!! Read header
    read(mtohc,'(a)',iostat=iosohc) header1
    read(mterr,'(a)',iostat=ioserr) header2
    if(iosohc.ne.0.or.ioserr.ne.0) then
      write(*,*) 'No corresponding year'
      stop
    endif
    read(header1(2:),'(i6)') year
    
!!! Check the year
    if(year.ne.iyear) then
      do j=1,jmx
        read(mtohc,fmt)
        read(mterr,fmt)
      enddo
      cycle
    endif

!!! Read missing value
    iptmiss=index(header1,'Missing=')
    read(header1(iptmiss+8:),'(f8.4)') xmiss
    
!!! Read OHC and error data
    do j=1,jmx
      read(mtohc,fmt) ohc(:,j)
      read(mterr,fmt) err(:,j)
    enddo
    exit
  enddo
  
  close(mtohc)
  close(mterr)
  
  
  !
  !
  ! The OHC anomalies are set to variable ohc(imx,jmx) and the error are set to variable err(imx,jmx)
  ! (1,1)   = (0.5E,89.5N)  , (2,1)   = (1.5E,89.5N) ,,, (360,1)   = (0.5W,89.5N)
  ! (1,2)   = (0.5E,88.5N)  , (2,2)   = (1.5E,88.5N) ,,, (360,2)   = (0.5W,88.5N)
  ! ,,,
  ! (1,180) = (0.5E,89.5S)  , (2,180) = (1.5E,89.5S) ,,, (360,180) = (0.5W,89.5S)
  !
  !
  !
  
  
  ! print (179.5E,0.5N) value
  write(*,*) ohc(180,90),err(180,90)
  
  stop
end program
